#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from airflow.models import DAG

from airflow.operators.bash_operator import BashOperator
from airflow.operators.dummy_operator import DummyOperator

from datetime import timedelta, datetime
import pendulum


# time zone을 한국 시간으로 설정하려면 pendulum 등의 library를 이용한다.
local_tz = pendulum.timezone("Asia/Seoul")


"""
DAG는 개별 task와 그 의존관계를 정의한 task의 집합체다.
처음에 DAG 객체를 정의하고 개별 task의 인수로 갖게 함으로써
공통된 인수(스케줄, 재시도 유무나 횟수 등)를 설정해 task 집합인 DAG를 정의하고 있다.
"""
# DAG객체에서 이용하는 공통된 인수를 정의한다.
default_args = {
    'owner': 'Airflow',
    'start_date': datetime(2019, 1, 1, tzinfo=pendulum.timezone('Asia/Seoul')),
    'depends_on_past': True,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

# DAG객체를 정의한다.
# 여기서는 정기 실행의 정의(schedule_interval)를 '1회만'으로 하고 있다.
dag = DAG(
    dag_id='sample_dag',
    default_args=default_args,
    schedule_interval='@once',
)

""" 여기서는 개별 task의 정의를 시작한다. """
# 우선, BashOperator를 사용하여 현재 시각을 표시한다.
operator_1 = BashOperator(
    task_id='operator_1',
    bash_command='echo {}'.format(datetime.now()),
    dag=dag,
)

# 다음으로, 워크플로를 보기 위한 도구로 더미 오퍼레이터를 정의한다.
# 더미 오퍼레이터는 자기 자신이 특별한 처리를 하지 않는다.
# 다만, 오퍼레이터 사이에 개입시킴으로써 의존관계의 정의에 유용한 경우가 있다. 여기서는 DAG를 이해하기 위한 도구로 사용한다.
operator_2 = DummyOperator(
    task_id='operator_2',
    trigger_rule='all_success',
    dag=dag,
)

# 2번째 더미 오퍼레이터다.
operator_3 = DummyOperator(
    task_id='operator_3',
    trigger_rule='all_success',
    dag=dag,
)

# 3번째 더미 오퍼레이터다
operator_4 = DummyOperator(
    task_id='operator_4',
    trigger_rule='all_success',
    dag=dag,
)

# 마지막으로 한 번 더 bash operator를 사용하여 현재 시각을 표시한다.
operator_5 = BashOperator(
    task_id='operator_5',
    bash_command='echo {}'.format(datetime.now()),
    dag=dag,
)

# 의존관계의 기술법은 여럿 있다.
# 여기서는 직감적으로 이해하기 쉽게 bit shift operator를 사용한 방법으로 기술하고 있다.
# operator_3, operator_4에 대해서는 리스트([]) 안에 쉼표 구분으로 기술하고 있는데 이것에 의해 두 갈래로 분기하여 분기 후 양쪽이 실행된다.
operator_1 >> operator_2 >> [operator_3, operator_4] >> operator_5
