#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from airflow.models import DAG

from airflow.operators.bash_operator import BashOperator
from airflow.operators.python_operator import BranchPythonOperator
from airflow.operators.dummy_operator import DummyOperator

import pendulum
import random
from datetime import timedelta, datetime

# time zone을 한국 시간으로 설정하려면 pendulum 등의 library를 이용한다.
local_tz = pendulum.timezone("Asia/Seoul")

"""
branch_python_operator.py는 처음에 BashOperator를 사용하여 현재 시각을 표시하고 다음으로 branching(분기)용 Python 함수를 정의한 후에 BranchPythonOperator에 의해 그 함수를 호출한다. 그런 다음 산출된 난수에 따라 의존하는 곳으로 분기하는(의존하는 곳이 하나만 실행되고 다른 분기점은 스킵된다) DAG다.
"""

# DAG 객체에서 이용하는 공통된 인수를 정의한다.
default_args = {
    'owner': 'Airflow',
    'start_date': datetime(2019, 1, 1, tzinfo=pendulum.timezone('Asia/Seoul')),
    'depends_on_past': True,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

# DAG 객체를 정의한다.
# 여기서는 정기 실행의 정의(schedule_interval)를 '한 번만'으로 한다.
dag = DAG(
    dag_id='branch_python_operator',
    default_args=default_args,
    schedule_interval='@once',
)

""" 여기서부터 개별 task의 정의를 시작한다. """
# 우선은 BashOperator를 사용하여 현재 시각을 표시한다.
operator_1 = BashOperator(
    task_id='operator_1',
    bash_command='echo {}'.format(datetime.now()),
    dag=dag,
)

# 다음으로 PythonOpeartor가 호출할 분기(branching)용 함수를 정의한다.
# 여기서는 난수(0.0 이상 1.0 미만의 float형 난수)를 발생시켜 값의 구간에 따라 의존하는 곳의 Operator를 분기시킨다.
def branching(rnd):
    if rnd >= 0 and rnd <= 0.2:
        return 'operator_3'
    elif rnd > 0.2 and rnd <= 0.4:
        return 'operator_4'
    elif rnd > 0.4 and rnd <= 0.6:
        return 'operator_5'
    elif rnd > 0.6 and rnd <= 0.8:
        return 'operator_6'
    else:
        return 'operator_7'

# 난수를 발생시킨다.
rnd = random.random()
# 다음으로 BranchPythonOperator를 사용하여 분기(branching)용 함수를 호출한다.
operator_2 = BranchPythonOperator(
    task_id='branch_python_operator',
    python_callable=branching,
    op_kwargs={'rnd': rnd},
    dag=dag,
)

# 분기하는 곳은 모두 더미 오퍼레이터로 한다.
operator_3 = DummyOperator(
    task_id='operator_3',
    trigger_rule='all_success',
    dag=dag,
)

operator_4 = DummyOperator(
    task_id='operator_4',
    trigger_rule='all_success',
    dag=dag,
)

operator_5 = DummyOperator(
    task_id='operator_5',
    trigger_rule='all_success',
    dag=dag,
)

operator_6 = DummyOperator(
    task_id='operator_6',
    trigger_rule='all_success',
    dag=dag,
)

operator_7 = DummyOperator(
    task_id='operator_7',
    trigger_rule='all_success',
    dag=dag,
)

# 마지막 오퍼레이터는, 분기한 오퍼레이터 중 하나가 성공하면 실행되는 더미 오퍼레이터로 한다.
operator_8 = DummyOperator(
    task_id='operator_8',
    trigger_rule='one_success',
    dag=dag,
)

# Operation의 의존관계를 정의한다.
operator_1 >> operator_2 >> [operator_3, operator_4,
                             operator_5, operator_6, operator_7] >> operator_8
